# @(#) $Id: VCFFile.pm,v 1.4 2002/05/30 11:56:19 nick Exp $
#	Copyright (C) 2002, Nick Andrew <nick@nick-andrew.net>
#	Distributed under the terms of the GNU Public License
#	See http://www.gnu.org/licenses/gpl.html
#
# A VCFFile is a file format containing VCalendar Objects.

package VCFFile;

use VAlarm;
use VCalendar;
use VEvent;
use VGeneric;
use VTodo;

sub new {
	my $class = shift;
	my $self = { @_ };

	bless $self, $class;
	return $self;
}

sub load_obj {
	my $self = shift;
	my $otype = shift;
	my $level = shift;

	my $o = { };

	$o->{type} = $otype;
	$o->{attrs} = { };
	$o->{contents} = [ ];

	my $line;

	# Read keys and values

	while (defined ($line = <F>)) {
		$line =~ s/\r//;
		chomp($line);

		# Deal with an apparent continuation line
		if ($line =~ /:$/) {
			my $nextline = <F>;
			chomp($nextline);
			$nextline =~ s/^ //;
			$line .= $nextline;
		}

		if ($line =~ /^([^:;]+)(;([^:]+))?:(.+)/) {
			my($key, $extend, $value) = ($1, $3, $4);

#			print STDERR "key is <$key> extend is <$extend> value is <$value>\n";
			if ($key eq 'BEGIN') {
				printf STDERR "%sBEGIN %s\n", ' ' x ($level * 4), $value;
				my $newo = $self->load_obj($value, $level + 1);
				push(@{$o->{contents}}, $newo);
				next;
			}

			if ($key eq 'END') {
				printf STDERR "%sEND %s\n", ' ' x ($level * 4), $value;
				if ($value ne $otype) {
					die "File error: expected END:$otype got END:$value";
				}

				my $obj;

				if ($otype eq 'VEVENT') {
					$obj = new VEvent();
				} elsif ($otype eq 'VTODO') {
					$obj = new VTodo();
				} elsif ($otype eq 'VCALENDAR') {
					$obj = new VCalendar();
				} elsif ($otype eq 'STANDARD') {
					$obj = new VGeneric();
				} elsif ($otype eq 'DAYLIGHT') {
					$obj = new VGeneric();
				} elsif ($otype eq 'VTIMEZONE') {
					$obj = new VGeneric();
				} elsif ($otype eq 'VALARM') {
					$obj = new VAlarm();
				} else {
					die "Unknown type: $otype";
				}

				$obj->{attrs} = $o->{attrs};
				$obj->{contents} = $o->{contents};

				return $obj;
			}

			$o->{attrs}->{$key} = $value;
		} else {
			# Ignore this line
		}
	}

	warn "Unable to load: ran off the end of file in $otype\n";

	return $o;
}

sub load {
	my $self = shift;
	my $path = shift;

	if (!open(F, "<$path")) {
		die "Unable to load $path: $!";
	}

	# Read BEGIN:VCALENDAR line

	my $line = <F>;
	$line =~ s/\r//;
	chomp($line);

	if ($line ne 'BEGIN:VCALENDAR') {
		die "Unable to load $path: file begins with ($line), expecting (BEGIN:VCALENDAR)";
	}

	my $o = $self->load_obj('VCALENDAR');

	# Move the vcalendar contents up in the object data structures

	$self->{type} = 'VCALENDAR';
	$self->{calendar} = $o;
	$self->{attrs} = $o->{attrs};
	$self->{contents} = $o->{contents};

}

1;
