# @(#) $Id: VCalendarObject.pm,v 1.4 2002/05/30 11:56:19 nick Exp $
#	Copyright (C) 2002, Nick Andrew <nick@nick-andrew.net>
#	Distributed under the terms of the GNU Public License
#	See http://www.gnu.org/licenses/gpl.html
#
#  A VCalendarObject is the superclass for all objects in the file.
#  This class defines common methods for them all.

package VCalendarObject;

use VCalendarRule;

sub new {
	my $class = shift;
	my $self = { @_ };

	bless $self, $class;
	return $self;
}

sub get_attr {
	my $self = shift;
	my $attr_name = shift;

	return $self->{attrs}->{$attr_name};
}

sub get_start_date {
	my $self = shift;

	my $dtstart = $self->{attrs}->{DTSTART};

	return undef if (!defined $dtstart);

	$dtstart =~ s/(\d\d\d\d)(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)/$1-$2-$3 $4:$5:$6/;

	return $dtstart;
}

sub get_start_time {
	my $self = shift;

	my $dtstart = $self->{attrs}->{DTSTART};

	return undef if (!defined $dtstart);

	if ($dtstart =~ /T(\d\d)(\d\d)(\d\d)/) {
		return "$1:$2:$3";
	}

	return "00:00:00";
}

sub get_summary {
	my $self = shift;
	return $self->{attrs}->{SUMMARY};
}

sub get_description {
	my $self = shift;
	return $self->{attrs}->{DESCRIPTION};
}

sub apply {
	my $self = shift;
	my $block = shift;

	&$block($self);

	foreach my $obj (@{$self->{contents}}) {
		$obj->apply($block);
	}
}

# Return 1 if an event start happens on the given date, else 0
# This is a stub to be overridden by all event type classes.

sub starts_on_date {
	return 0;
}

1;
