# @(#) $Id: VEvent.pm,v 1.2 2002/05/30 11:56:19 nick Exp $
#	Copyright (C) 2002, Nick Andrew <nick@nick-andrew.net>
#	Distributed under the terms of the GNU Public License
#	See http://www.gnu.org/licenses/gpl.html
#
#  A VEvent represents something which starts at a particular time
#  and has a particular duration. A VEvent repeats if it has an
#  RRULE attribute, in which case the event can occur on multiple
#  days.

package VEvent;

use VCalendarObject;

use vars qw/@ISA/;

@ISA = qw/VCalendarObject/;

# Return 1 if an event start happens on the given date, else 0
#

sub starts_on_date {
	my $self = shift;
	my $target_date = shift;

	my $start_date = $self->get_start_date();

	return 0 if (!defined $start_date);

#	print STDERR "My date is $start_date\n";

	$start_date =~ s/ .*//;

	return 0 if ($start_date gt $target_date);
	return 1 if ($start_date eq $target_date);

	# Maybe it's a repeating one!

	my $rrule = $self->get_attr('RRULE');
	return 0 if (!defined $rrule);

	# At this point we should check for attribute EXDATE which is
	# an exclusion of an instance of a recurring event

	if ($self->get_attr('EXDATE')) {
		my $exdate = $self->get_attr('EXDATE');
		die "Unable to check an Event which contains an EXDATE: $exdate\n";
	}

	my $rrule_obj;

	if (!exists $self->{rrule_obj}) {
		$self->{rrule_obj} = new VCalendarRule($rrule);
	}

	$rrule_obj = $self->{rrule_obj};

	if ($rrule_obj->applies_to_date($target_date)) {
#		print STDERR "Rule applies to $target_date\n";
		return 1;
	}

	return 0;
}

1;
