#!/usr/bin/perl
# @(#) $Id: vcparse.pl,v 1.5 2002/05/30 11:56:20 nick Exp $
#	Copyright (C) 2002, Nick Andrew <nick@nick-andrew.net>
#	Distributed under the terms of the GNU Public License
#	See http://www.gnu.org/licenses/gpl.html
#
#	@(#) vcparse.pl filename [target_date]
#
#	Parse the requested vcalendar file, and find events which start
#	on the requested date (or today, if target_date not supplied).
#	Output a concise (< 160 chars long) summary of the events to STDOUT.


use VCFFile;

my $file = shift @ARGV;
my $today = shift @ARGV;

my $f = new VCFFile();

$f->load($file);

# print "Dumping attributes:\n";
# rec_dump($f->{attrs}, 0);

# print "Dumping contents:\n";
# rec_dump($f->{contents}, 0);

my $cal = $f->{calendar};

my @list;

if (!defined $today) {
	my($sec,$min,$hour,$mday,$mon,$year) = localtime(time());
	$mon++; $year += 1900;
	$today = sprintf "%d-%02d-%02d", $year, $mon, $mday;
}

# Recursively look for objects which start on the given date

# print STDERR "Today is $today\n";

$cal->apply (sub {
	if ($_[0]->starts_on_date($today)) {
		push(@list, $_[0]);
	}
});


if (@list) {

	my @sl = sort { $a->get_start_time() cmp $b->get_start_time() } (@list);
	my @words;

	foreach my $o (@sl) {
		my $sd = $o->get_start_date();
		# yyyy-mm-dd hh:mm:ss
		my $st = substr($sd, 11, 5);
		my $summary = $o->get_summary() || $o->get_description();
		push(@words, $st, $summary);
	}

	my $s1 = "Today's schedule: ";

	word_balance(159 - length($s1), \@words);
	
	print $s1, "@words\n";
}


exit(0);

# word_balance($length, [strings, ...])
# Make sure that the given strings will fit into the required length,
# by recursively chopping words from the longest strings until it all
# fits.

sub word_balance {
	my $max_l = shift;
	my $lr = shift;

	# Count total length + 1 space between each word
	my $l = 0;
	my $first = 1;
	my $longest = 0;

	foreach (@$lr) {
		if ($first) {
			$first = 0;
		} else {
			$l++;	# 1 space
		}

		my $len = length($_);
		$l += $len;
		if ($len > $longest) {
			$longest = $len;
		}
	};

	return $l if ($l <= $max_l);

	# Okay, we have too much! Try to remove some stuff from the end...
	#
	# Remove one subword from all words with longest length and recurse

	foreach (@$lr) {
		my $len = length($_);
		if ($len == $longest) {
			my $pos = rindex($_, ' ');
			if ($pos == -1) {
				# Oops, no words, so truncate 5 chars
				$_ = substr($_, 0, $len - 5);
			} else {
				$_ = substr($_, 0, $pos);
			}
		}
	}

	my $revised_l = word_balance($max_l, $lr);

	return $revised_l;
}

sub rec_dump {
	my $r = shift;
	my $level = shift;

	if (!ref $r) {
		print $r, "\n";
		return;
	}

	my $t = ref $r;

	if ($t =~ /^HASH/ || UNIVERSAL::isa($r, "HASH")) {
		print "{\n";
		foreach my $k (sort (keys %$r)) {
			print "\t" x ($level + 1), $k, " => ";
			rec_dump($r->{$k}, $level + 1);
		}
		print "\t" x $level, "}", "\n";

		return;
	}

	if ($t =~ /^ARRAY/) {
		print "[", "\n";
		foreach my $v (@$r) {
			print "\t" x ($level + 1);
			rec_dump($v, $level + 1);
		}
		print "\t" x $level, "]", "\n";

		return;
	}

	die "Urk: $t";
}

